@@ -192,10 +192,11 @@ typedef enum {
 
 
 /**
- * These ISO, shutter speed, aperture, etc. settings are correct for the 
- * EOS 5D; unsure about other cameras.
+ * These ISO, shutter speed, aperture, etc. settings are correct for the
+ * EOS 5D and some for the G1; unsure about other cameras.
  */
 typedef enum {
+	ISO_50 = 0x40,
 	ISO_100 = 0x48,
 	ISO_125 = 0x4b,
 	ISO_160 = 0x4d,
@@ -208,7 +209,8 @@ typedef enum {
 	ISO_800 = 0x60,
 	ISO_1000 = 0x63,
 	ISO_1250 = 0x65,
-	ISO_1600 = 0x68
+	ISO_1600 = 0x68,
+	ISO_3200 = 0x70
 } canonIsoState;
 
 struct canonIsoStateStruct {
@@ -216,7 +218,19 @@ struct canonIsoStateStruct {
 	char *label;
 };
 
+struct canonShootingModeStateStruct {
+	unsigned char value;
+	char *label;
+};
+
 typedef enum {
+	APERTURE_F1_2 = 0x0d,
+	APERTURE_F1_4 = 0x10,
+	APERTURE_F1_6 = 0x13,
+	APERTURE_F1_8 = 0x15,
+	APERTURE_F2_0 = 0x18,
+	APERTURE_F2_2 = 0x1b,
+	APERTURE_F2_5 = 0x1d,
 	APERTURE_F2_8 = 0x20,
 	APERTURE_F3_2 = 0x23,
 	APERTURE_F3_5 = 0x25,
@@ -235,7 +249,10 @@ typedef enum {
 	APERTURE_F16 = 0x48,
 	APERTURE_F18 = 0x4b,
 	APERTURE_F20 = 0x4d,
-	APERTURE_F22 = 0x50
+	APERTURE_F22 = 0x50,
+	APERTURE_F25 = 0x53,
+	APERTURE_F29 = 0x55,
+	APERTURE_F32 = 0x58,
 } canonApertureState;
 
 struct canonApertureStateStruct {
@@ -296,7 +313,10 @@ typedef enum {
 	SHUTTER_SPEED_1_2000 = 0x90,
 	SHUTTER_SPEED_1_2500 = 0x93,
 	SHUTTER_SPEED_1_3200 = 0x95,
-	SHUTTER_SPEED_1_4000 = 0x98
+	SHUTTER_SPEED_1_4000 = 0x98,
+	SHUTTER_SPEED_1_5000 = 0x9a,
+	SHUTTER_SPEED_1_6400 = 0x9d,
+	SHUTTER_SPEED_1_8000 = 0xA0
 } canonShutterSpeedState;
 
 struct canonShutterSpeedStateStruct {
@@ -305,24 +325,25 @@ struct canonShutterSpeedStateStruct {
 };
 
 typedef enum {
-	RESOLUTION_RAW                        = 0,
-	RESOLUTION_RAW_AND_LARGE_FINE_JPEG, 
-	RESOLUTION_RAW_AND_LARGE_NORMAL_JPEG, 
-	RESOLUTION_RAW_AND_MEDIUM_FINE_JPEG, 
-	RESOLUTION_RAW_AND_MEDIUM_NORMAL_JPEG,
-	RESOLUTION_RAW_AND_SMALL_FINE_JPEG,
-	RESOLUTION_RAW_AND_SMALL_NORMAL_JPEG,
-	RESOLUTION_LARGE_FINE_JPEG,
-	RESOLUTION_LARGE_NORMAL_JPEG,
-	RESOLUTION_MEDIUM_FINE_JPEG,
-	RESOLUTION_MEDIUM_NORMAL_JPEG,
-	RESOLUTION_SMALL_FINE_JPEG,
-	RESOLUTION_SMALL_NORMAL_JPEG
-} canonResolutionState;
+	IMAGE_FORMAT_RAW                        = 0,
+	IMAGE_FORMAT_RAW_2,
+	IMAGE_FORMAT_RAW_AND_LARGE_FINE_JPEG,
+	IMAGE_FORMAT_RAW_AND_LARGE_NORMAL_JPEG,
+	IMAGE_FORMAT_RAW_AND_MEDIUM_FINE_JPEG,
+	IMAGE_FORMAT_RAW_AND_MEDIUM_NORMAL_JPEG,
+	IMAGE_FORMAT_RAW_AND_SMALL_FINE_JPEG,
+	IMAGE_FORMAT_RAW_AND_SMALL_NORMAL_JPEG,
+	IMAGE_FORMAT_LARGE_FINE_JPEG,
+	IMAGE_FORMAT_LARGE_NORMAL_JPEG,
+	IMAGE_FORMAT_MEDIUM_FINE_JPEG,
+	IMAGE_FORMAT_MEDIUM_NORMAL_JPEG,
+	IMAGE_FORMAT_SMALL_FINE_JPEG,
+	IMAGE_FORMAT_SMALL_NORMAL_JPEG
+} canonImageFormatState;
 
 
-struct canonResolutionStateStruct {
-	canonResolutionState value;
+struct canonImageFormatStateStruct {
+	canonImageFormatState value;
 	char *label;
 	unsigned char res_byte1;
 	unsigned char res_byte2;
@@ -341,18 +362,66 @@ struct canonFocusModeStateStruct {
 	char *label;
 };
 
+/**
+ * canonFlashMode:
+ * @FLASH_MODE_OFF: The flash does not fire
+ * @FLASH_MODE_ON: The flash does fire (compulsory)
+ * @FLASH_MODE_AUTO: The flash fires if necessary
+ */
+typedef enum {
+	FLASH_MODE_OFF = 0,
+	FLASH_MODE_ON,
+	FLASH_MODE_AUTO
+} canonFlashMode;
+
+struct canonFlashModeStateStruct {
+	canonFlashMode value;
+	char* label;
+};
+
+/**
+ * canonBeepMode:
+ * @BEEP_OFF: The camera does not beep after focusing the image
+ * @BEEP_ON: The camera emits an audible beep after focusing the image
+ */
+typedef enum {
+	BEEP_OFF = 0x00,
+	BEEP_ON = 0x01
+} canonBeepMode;
+
+struct canonBeepModeStateStruct {
+	canonBeepMode value;
+	char* label;
+};
+
+struct canonZoomLevelStateStruct {
+	unsigned char value;
+	char* label;
+};
+
+struct canonExposureBiasStateStruct {
+	unsigned char value;
+	char* label;
+};
+
+
 /* Size of the release parameter block */
 #define RELEASE_PARAMS_LEN  0x2f
 
 /* These indexes are byte offsets into the release parameter data */
-#define RESOLUTION_1_INDEX  0x01
-#define RESOLUTION_2_INDEX  0x02
-#define RESOLUTION_3_INDEX  0x03
-#define BEEP_INDEX          0x07
+#define IMAGE_FORMAT_1_INDEX  0x01
+#define IMAGE_FORMAT_2_INDEX  0x02
+#define IMAGE_FORMAT_3_INDEX  0x03
+#define SELF_TIMER_1_INDEX  0x04 /* Currently not used */
+#define SELF_TIMER_2_INDEX  0x05 /* Currently not used */
+#define FLASH_INDEX         0x06
+#define BEEP_INDEX          0x07 
 #define FOCUS_MODE_INDEX    0x12
 #define ISO_INDEX           0x1a
 #define APERTURE_INDEX      0x1c
 #define SHUTTERSPEED_INDEX  0x1e
+#define EXPOSUREBIAS_INDEX  0x20
+#define SHOOTING_MODE_INDEX 0x08
 
 /**
  * canonCaptureSizeClass:
@@ -425,6 +494,9 @@ struct _CameraPrivateLibrary
 
 	char *cached_drive;	/* usually something like C: */
 	int cached_ready;       /* whether the camera is ready to rock */
+
+	unsigned char *directory_state;	/* directory content state for wait_for_event */
+
 	long image_key, thumb_length, image_length; /* For immediate download of captured image */
 	long image_b_key, image_b_length; /* For immediate download of secondary captured image */
 	int capture_step;	/* To record progress in interrupt
@@ -564,7 +636,7 @@ typedef enum {
 #define GP_PORT_DEFAULT_RETURN_INTERNAL(return_statement) \
 		default: \
 			gp_context_error (context, _("Don't know how to handle " \
-					     "camera->port->type value %i aka 0x%x" \
+					     "camera->port->type value %i aka 0x%x " \
 					     "in %s line %i."), camera->port->type, \
 					     camera->port->type, __FILE__, __LINE__); \
 			return_statement; \
@@ -619,14 +691,17 @@ int canon_int_list_directory (Camera *camera, const char *folder, CameraList *li
 
 int canon_int_get_file(Camera *camera, const char *name, unsigned char **data, unsigned int *length, GPContext *context);
 int canon_int_get_thumbnail(Camera *camera, const char *name, unsigned char **retdata, unsigned int *length, GPContext *context);
-int canon_int_put_file(Camera *camera, CameraFile *file, char *destname, char *destpath, GPContext *context);
+int canon_int_put_file(Camera *camera, CameraFile *file, const char *filename, const char *destname, const char *destpath, GPContext *context);
+int canon_int_wait_for_event (Camera *camera, int timeout, CameraEventType *eventtype, void **eventdata, GPContext *context);
 int canon_int_set_file_attributes(Camera *camera, const char *file, const char *dir, canonDirentAttributeBits attrs, GPContext *context);
 int canon_int_delete_file(Camera *camera, const char *name, const char *dir, GPContext *context);
 int canon_int_set_shutter_speed(Camera *camera, canonShutterSpeedState shutter_speed, GPContext *context);
 int canon_int_set_iso(Camera *camera, canonIsoState iso, GPContext *context);
+int canon_int_set_shooting_mode (Camera *camera, unsigned char shooting_mode, GPContext *context);
 int canon_int_set_aperture(Camera *camera, canonApertureState aperture, GPContext *context);
+int canon_int_set_exposurebias(Camera *camera, unsigned char expbias, GPContext *context);
 int canon_int_set_focus_mode (Camera *camera, canonFocusModeState focus_mode, GPContext *context);
-int canon_int_set_resolution (Camera *camera, unsigned char res_byte1, unsigned char res_byte2, unsigned char res_byte3, GPContext *context);
+int canon_int_set_image_format (Camera *camera, unsigned char res_byte1, unsigned char res_byte2, unsigned char res_byte3, GPContext *context);
 int canon_serial_off(Camera *camera);
 int canon_int_get_time(Camera *camera, time_t *camera_time, GPContext *context);
 int canon_int_set_time(Camera *camera, time_t date, GPContext *context);
@@ -635,12 +710,17 @@ int canon_int_identify_camera(Camera *camera, GPContext *context);
 int canon_int_set_owner_name(Camera *camera, const char *name, GPContext *context);
 int canon_int_start_remote_control(Camera *camera, GPContext *context);
 int canon_int_end_remote_control(Camera *camera, GPContext *context);
+int canon_int_set_beep(Camera *camera, canonBeepMode beep_mode, GPContext *context);
+int canon_int_set_flash(Camera *camera, canonFlashMode flash_mode, GPContext *context);
+int canon_int_set_zoom(Camera *camera, unsigned char zoom_level, GPContext *context);
 
 /*
  * introduced for capturing
  */
 int canon_int_get_release_params (Camera *camera, GPContext *context);
 
+void canon_int_find_new_image ( Camera *camera, unsigned char *initial_state, unsigned char *final_state, CameraFilePath *path );
+
 
 /* path conversion - needs drive letter, and therefore cannot be moved
  * to util.c */
