@@ -117,7 +117,7 @@ static struct {
 
 	{"Nikon", "CoolPix 4300 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x010e,
 						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
-							&cp880_cam_desc},
+							&cp4300_cam_desc},
 	{"Nikon", "CoolPix 3500 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x0110,
 						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
 							&cp880_cam_desc},
@@ -205,11 +205,16 @@ static struct {
 				SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 	{"Olympus", "fe-200", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x109,
 				SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
+	{"Olympus", "E-520", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x110,
+				SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 
 	{"Panasonic", "Coolshot NV-DCF5E", SIERRA_MODEL_DEFAULT, 0, 0, 0, NULL },
 
 	{"Pentax", "Optio 450", SIERRA_MODEL_DEFAULT, 0x0a17,0x0007, SIERRA_WRAP_USB_PENTAX, NULL },
 	{"Pentax", "Optio 33WR", SIERRA_MODEL_DEFAULT, 0x0a17,0x0009, SIERRA_WRAP_USB_PENTAX, NULL },
+	/* has only very bare reactions
+	{"Pentax", "Optio W90 (Sierra Mode)", SIERRA_MODEL_DEFAULT, 0x0a17,0x00f6, SIERRA_WRAP_USB_PENTAX, NULL },
+	 */
 
 	{"Polaroid", "PDC 640", 	SIERRA_MODEL_DEFAULT,	0, 0, 0, NULL },
 	{"Polaroid", "PDC 2300Z", SIERRA_MODEL_DEFAULT, 0x0546, 0x0daf,
@@ -269,8 +274,12 @@ int camera_abilities (CameraAbilitiesList *list)
 		a.status = GP_DRIVER_STATUS_PRODUCTION;
 		a.port     = GP_PORT_SERIAL;
 		if ((sierra_cameras[x].usb_vendor  > 0) &&
-		    (sierra_cameras[x].usb_product > 0))
-			a.port |= GP_PORT_USB;
+		    (sierra_cameras[x].usb_product > 0)) {
+			if (sierra_cameras[x].flags & (SIERRA_WRAP_USB_OLYMPUS|SIERRA_WRAP_USB_PENTAX|SIERRA_WRAP_USB_NIKON))
+			    a.port |= GP_PORT_USB_SCSI;
+                        else
+			    a.port |= GP_PORT_USB;
+		}
 		a.speed[0] = 9600;
 		a.speed[1] = 19200;
 		a.speed[2] = 38400;
@@ -350,11 +359,6 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	info->audio.fields   = GP_FILE_INFO_NONE;
 	info->file.permissions = GP_FILE_PERM_READ;
 
-	/* Name of image */
-	strncpy (info->file.name, filename, sizeof (info->file.name) - 1);
-	info->file.name[sizeof (info->file.name) - 1] = '\0';
-	info->file.fields |= GP_FILE_INFO_NAME;
-
 	/*
 	 * Get information about this image. Don't make this fatal as
 	 * at least the "Agfa ePhoto307" doesn't support this command.
@@ -477,6 +481,7 @@ camera_start (Camera *camera, GPContext *context)
 		break;
 
 	case GP_PORT_USB:
+	case GP_PORT_USB_SCSI:
 		CHECK (gp_port_set_timeout (camera->port, 5000));
 		break;
 	default:
@@ -733,22 +738,22 @@ camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
 }
 
 static int
-put_file_func (CameraFilesystem * fs, const char *folder, CameraFile * file, void *data, GPContext *context)
+put_file_func (CameraFilesystem * fs, const char *folder, const char *filename,
+	CameraFileType type, CameraFile * file, void *data, GPContext *context)
 {
 	Camera *camera = data;
 	char *picture_folder;
 	int ret;
 	const char *data_file;
 	long data_size;
-	const char *filename;
 	int available_memory;
 
-	gp_file_get_name(file, &filename);
-	
-
 	GP_DEBUG ("*** put_file_func");
 	GP_DEBUG ("*** folder: %s", folder);
 	GP_DEBUG ("*** filename: %s", filename);
+
+	if (type != GP_FILE_TYPE_NORMAL)
+		return GP_ERROR_BAD_PARAMETERS;
 	
 	/* Check the size */
 	CHECK (gp_file_get_data_and_size (file, &data_file, &data_size));
@@ -1590,17 +1595,17 @@ camera_get_config_epson (Camera *camera, CameraWidget **window, GPContext *conte
 	ret = sierra_get_int_register (camera, 1, &value, context);
         if (ret >= 0) {
 		gp_widget_new (GP_WIDGET_RADIO, _("Resolution"), &child);
-		gp_widget_add_choice (child, _("Standard"));
-		gp_widget_add_choice (child, _("Fine"));
-		gp_widget_add_choice (child, _("SuperFine"));
+		gp_widget_add_choice (child, _("standard"));
+		gp_widget_add_choice (child, _("fine"));
+		gp_widget_add_choice (child, _("superfine"));
 		gp_widget_add_choice (child, _("HyPict"));
 		
                 switch (value) {
-		case 1: strcpy (t, _("Standard"));
+		case 1: strcpy (t, _("standard"));
 			break;
-                case 2: strcpy (t, _("Fine"));
+                case 2: strcpy (t, _("fine"));
 			break;
-                case 3: strcpy (t, _("SuperFine"));
+                case 3: strcpy (t, _("superfine"));
 			break;
                 case 34: strcpy (t, _("HyPict"));
 			break;
@@ -1801,11 +1806,11 @@ camera_set_config_epson (Camera *camera, CameraWidget *window, GPContext *contex
 	if ((gp_widget_get_child_by_label (window, _("Resolution"), &child)
 	     >= 0) && gp_widget_changed (child)) {
 		gp_widget_get_value (child, &value);
-		if (strcmp (value, _("Standard")) == 0) {
+		if (strcmp (value, _("standard")) == 0) {
 			i = 1;
-		} else if (strcmp (value, _("Fine")) == 0) {
+		} else if (strcmp (value, _("fine")) == 0) {
 			i = 2;
-		} else if (strcmp (value, _("SuperFine")) == 0) {
+		} else if (strcmp (value, _("superfine")) == 0) {
 			i = 3;
 		} else if (strcmp (value, _("HyPict")) == 0) {
 			i = 34;
@@ -1931,40 +1936,84 @@ camera_summary (Camera *camera, CameraText *summary, GPContext *c)
 	/* Get all the string-related info */
 
 	if (sierra_get_string_register (camera, 27, 0, NULL, t, &v, c) >= 0)
-		sprintf (buf, _("%sCamera Model: %s\n"), buf, t);
+		sprintf (buf+strlen(buf), _("Camera Model: %s\n"), t);
 	if (sierra_get_string_register (camera, 48, 0, NULL, t, &v, c) >= 0)
-		sprintf (buf, _("%sManufacturer: %s\n"), buf, t);
+		sprintf (buf+strlen(buf), _("Manufacturer: %s\n"), t);
 	if (sierra_get_string_register (camera, 22, 0, NULL, t, &v, c) >= 0)
-		sprintf (buf, _("%sCamera ID: %s\n"), buf, t);
+		sprintf (buf+strlen(buf), _("Camera ID: %s\n"), t);
 	if (sierra_get_string_register (camera, 25, 0, NULL, t, &v, c) >= 0)
-		sprintf (buf, _("%sSerial Number: %s\n"), buf, t);
+		sprintf (buf+strlen(buf), _("Serial Number: %s\n"), t);
 	if (sierra_get_string_register (camera, 26, 0, NULL, t, &v, c) >= 0)
-		sprintf (buf, _("%sSoftware Rev.: %s\n"), buf, t);
+		sprintf (buf+strlen(buf), _("Software Rev.: %s\n"), t);
 
 	/* Get all the integer information */
 	if (camera->pl->flags & SIERRA_NO_REGISTER_40) {
 		if (sierra_get_int_register(camera, 10, &v, c) >= 0)
-			sprintf (buf, _("%sFrames Taken: %i\n"), buf, v);
+			sprintf (buf+strlen(buf), _("Frames Taken: %i\n"), v);
 	} else {
 		if (sierra_get_int_register(camera, 40, &v, c) >= 0)
-			sprintf (buf, _("%sFrames Taken: %i\n"), buf, v);
+			sprintf (buf+strlen(buf), _("Frames Taken: %i\n"), v);
 	}
 	if (sierra_get_int_register(camera, 11, &v, c) >= 0)
-		sprintf (buf, _("%sFrames Left: %i\n"), buf, v);
+		sprintf (buf+strlen(buf), _("Frames Left: %i\n"), v);
 	if (sierra_get_int_register(camera, 16, &v, c) >= 0)
-		sprintf (buf, _("%sBattery Life: %i\n"), buf, v);
+		sprintf (buf+strlen(buf), _("Battery Life: %i\n"), v);
 	if (sierra_get_int_register(camera, 28, &v, c) >= 0)
-		sprintf (buf, _("%sMemory Left: %i bytes\n"), buf, v);
+		sprintf (buf+strlen(buf), _("Memory Left: %i bytes\n"), v);
 
 	/* Get date */
 	if (sierra_get_int_register (camera, 2, &v, c) >= 0) {
 		time_t vtime = v;
-		sprintf (buf, _("%sDate: %s"), buf, ctime (&vtime));
+		sprintf (buf+strlen(buf), _("Date: %s"), ctime (&vtime));
+	}
+	strcpy (summary->text, buf);
+	return (camera_stop (camera, c));
+}
+
+static int
+storage_info_func (CameraFilesystem *fs,
+                CameraStorageInformation **sinfos,
+                int *nrofsinfos,
+                void *data, GPContext *c
+) {
+        Camera 			*camera = data;
+        CameraStorageInformation*sif;
+	int			v;
+	char			t[1024];
+
+	GP_DEBUG ("*** sierra storage_info");
+
+	CHECK (camera_start (camera, c));
+
+	sif = malloc(sizeof(CameraStorageInformation));
+	if (!sif)
+		return GP_ERROR_NO_MEMORY;
+	*sinfos = sif;
+	*nrofsinfos = 1;
+
+	sif->fields  = GP_STORAGEINFO_BASE;
+	strcpy (sif->basedir, "/");
+	sif->fields |= GP_STORAGEINFO_STORAGETYPE;
+	sif->type    = GP_STORAGEINFO_ST_REMOVABLE_RAM;
+	sif->fields |= GP_STORAGEINFO_ACCESS;
+	sif->access  = GP_STORAGEINFO_AC_READWRITE;
+	sif->fields |= GP_STORAGEINFO_FILESYSTEMTYPE;
+	sif->fstype  = GP_STORAGEINFO_FST_DCF;
+
+	if (sierra_get_string_register (camera, 25, 0, NULL, t, &v, c) >= 0) {
+		sif->fields |= GP_STORAGEINFO_LABEL;
+		strcpy (sif->label, t);
 	}
 
-	strcpy (summary->text, buf);
-
-	return (camera_stop (camera, c));
+	if (sierra_get_int_register(camera, 11, &v, c) >= 0) {
+		sif->fields |= GP_STORAGEINFO_FREESPACEIMAGES;
+		sif->freeimages = v;
+	}
+	if (sierra_get_int_register(camera, 28, &v, c) >= 0) {
+		sif->fields |= GP_STORAGEINFO_FREESPACEKBYTES;
+		sif->freekbytes = v/1024;
+	}
+	return camera_stop (camera, c);
 }
 
 static int
@@ -1998,17 +2047,15 @@ camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 		    "(1) Camera Configuration:\n"
 		    "    A value of 0 will take the default one (auto).\n"
 		    "(2) Olympus C-3040Z (and possibly also the C-2040Z\n"
-		    "    and others) have a USB PC Control mode. In order\n"
-		    "    to use this mode, the camera must be switched\n"
-		    "    into 'USB PC control mode'. To get to the menu\n"
-		    "    for switching modes, turn on the camera, open\n"
+		    "    and others) have a USB PC Control mode. To switch\n"
+		    "    to this mode, turn on the camera, open\n"
 		    "    the memory card access door and then press and\n"
 		    "    hold both of the menu and LCD buttons until the\n"
 		    "    camera control menu appears. Set it to ON.\n"
 		    "(3) If you switch the 'LCD mode' to 'Monitor' or\n"
 		    "    'Normal', don't forget to switch it back to 'Off'\n"
 		    "    before disconnecting. Otherwise you cannot use\n"
-		    "    the camera's buttons. If you end up with this\n"
+		    "    the camera's buttons. If you end up in this\n"
 		    "    state, you should reconnect the camera to the\n"
 		    "    PC and switch LCD to 'Off'."));
 		break;
@@ -2026,7 +2073,7 @@ camera_about (Camera *camera, CameraText *about, GPContext *context)
 		_("sierra SPARClite library\n"
 		"Scott Fritzinger <scottf@unr.edu>\n"
 		"Support for sierra-based digital cameras\n"
-		"including Olympus, Nikon, Epson, Pentax, and others.\n"
+		"including Olympus, Nikon, Epson, and Pentax.\n"
 		"\n"
 		"Thanks to Data Engines (www.dataengines.com)\n"
 		"for the use of their Olympus C-3030Z for USB\n"
@@ -2086,6 +2133,7 @@ static CameraFilesystemFuncs fsfuncs = {
 	.put_file_func = put_file_func,
 	.del_file_func = delete_file_func,
 	.delete_all_func = delete_all_func,
+	.storage_info_func = storage_info_func,
 };
 
 int
@@ -2187,6 +2235,7 @@ camera_init (Camera *camera, GPContext *context)
                 break;
 
         case GP_PORT_USB:
+        case GP_PORT_USB_SCSI:
 
                 /* Test if we have usb information */
                 if (vendor == 0) {
