@@ -101,12 +101,12 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 
 	bytes = ((8*(1+numpics))+0x3f) & ~0x3f;
 	xbuf = malloc(bytes);
-	ret = gp_port_usb_msg_read( camera->port,BLINK2_GET_DIR,0x03,0,buf,1 );
+	ret = gp_port_usb_msg_read( camera->port,BLINK2_GET_DIR,0x03,0,(char*)buf,1 );
 	if (ret < GP_OK)  {
 		free(xbuf);
 		return ret;
 	}
-	ret = gp_port_read( camera->port, xbuf, bytes);
+	ret = gp_port_read( camera->port, (char*)xbuf, bytes);
 	if (ret < GP_OK) {
 		free(xbuf);
 		return ret;
@@ -149,13 +149,13 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		free(xbuf);
 		return GP_ERROR_NO_MEMORY;
 	}
-	ret = gp_port_usb_msg_read( camera->port,BLINK2_GET_DIR,0x03,0,buf,1 );
+	ret = gp_port_usb_msg_read( camera->port,BLINK2_GET_DIR,0x03,0,(char*)buf,1 );
 	if (ret < GP_OK) {
 		free(addrs);
 		free(xbuf);
 		return ret;
 	}
-	ret = gp_port_read (camera->port, xbuf, bytes);
+	ret = gp_port_read (camera->port, (char*)xbuf, bytes);
 	if (ret < GP_OK) {
 		free(addrs);
 		free(xbuf);
@@ -182,15 +182,14 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		free(addrs);
                 return image_no;
 	}
-        gp_file_set_name (file, filename);
         switch (type) {
         case GP_FILE_TYPE_NORMAL:
 #ifdef HAVE_LIBJPEG
 {
 		char *convline,*convline2,*rawline;
 		unsigned char	*jpegdata;
-		unsigned int start, len, i, pitch;
-		int curread, ret;
+		unsigned int start, len, pitch;
+		int curread;
 		struct jpeg_decompress_struct	dinfo;
 		struct jpeg_source_mgr		xjsm;
 		struct jpeg_error_mgr		jerr;
@@ -215,14 +214,16 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		buf[5] = (len >> 16) & 0xff;
 		buf[6] = (len >>  8) & 0xff;
 		buf[7] =  len        & 0xff;
-		result = gp_port_usb_msg_write (camera->port,BLINK2_GET_MEMORY,0x03,0,buf,8);
-		if (result < GP_OK)
+		result = gp_port_usb_msg_write (camera->port,BLINK2_GET_MEMORY,0x03,0,(char*)buf,8);
+		if (result < GP_OK) {
+			free (jpegdata);
 			break;
+		}
 		len	*= 8;
 		curread  = 0;
 		do {
 			int res;
-			res = gp_port_read (camera->port, jpegdata+curread, len );
+			res = gp_port_read (camera->port, (char*)(jpegdata+curread), len );
 			if (res < GP_OK) {
 				result = GP_OK;
 				break;
@@ -242,8 +243,11 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		jpeg_create_decompress(&dinfo);
 		dinfo.src		= &xjsm;
 		ret = jpeg_read_header(&dinfo,TRUE);
-		if (ret != JPEG_HEADER_OK)
+		if (ret != JPEG_HEADER_OK) {
+			jpeg_destroy_decompress(&dinfo);
+			free (jpegdata);
 			break;
+		}
 		dinfo.out_color_space = JCS_RGB;
 		jpeg_start_decompress(&dinfo);
 
@@ -313,7 +317,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		buf[5] = (len >> 16) & 0xff;
 		buf[6] = (len >>  8) & 0xff;
 		buf[7] =  len        & 0xff;
-		result = gp_port_usb_msg_write(camera->port,BLINK2_GET_MEMORY,0x03,0,buf,8);
+		result = gp_port_usb_msg_write(camera->port,BLINK2_GET_MEMORY,0x03,0,(char*)buf,8);
 		if (result < GP_OK)
 			break;
 		len = len*8;
@@ -375,7 +379,7 @@ camera_capture (Camera *camera, CameraCaptureType type, CameraFilePath *path,
 		if (ret < GP_OK)
 			return ret;
 		sleep(1);
-	} while (buf[1] == 0x00);
+	} while (buf[0] == 0x00);
 
 	ret = blink2_getnumpics (camera->port, context, &numpics);
 	if (ret < GP_OK)
@@ -440,6 +444,7 @@ camera_init (Camera *camera, GPContext *context)
 
 	gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 	gp_port_get_settings( camera->port, &settings);
+	settings.usb.config = 1;
 	settings.usb.interface = 0;
 	settings.usb.altsetting = 0;
 	ret = gp_port_set_settings (camera->port, settings);
