@@ -79,25 +79,38 @@ gp_port_library_type (void)
 	return (GP_PORT_USB);
 }
 
-/* FIXME: timeout this cached information */
-static time_t devslastchecked = 0;
-static int nrofdevs = 0;
-static libusb_device **devs = NULL;
+static time_t gp_devslastchecked = 0;
+static int gp_nrofdevs = 0;
+static struct libusb_device_descriptor	*gp_descs;
+static libusb_device			**gp_devs;
 
 static ssize_t
-load_devicelist (void) {
+load_devicelist (libusb_context *ctx) {
 	time_t	xtime;
 
 	time(&xtime);
-	if (xtime != devslastchecked) {
-		if (nrofdevs)
-			libusb_free_device_list (devs, 0);
-		nrofdevs = 0;
-		devs = NULL;
+	if (xtime != gp_devslastchecked) {
+		if (gp_nrofdevs)
+			libusb_free_device_list (gp_devs, 1);
+		free (gp_descs);
+		gp_nrofdevs = 0;
+		gp_devs = NULL;
+		gp_descs = NULL;
 	}
-	if (!nrofdevs)
-		nrofdevs = libusb_get_device_list (NULL, &devs);
-	return nrofdevs;
+	if (!gp_nrofdevs) {
+		int 	i;
+
+		gp_nrofdevs = libusb_get_device_list (ctx, &gp_devs);
+		gp_descs = malloc (sizeof(gp_descs[0])*gp_nrofdevs);
+		for (i=0;i<gp_nrofdevs;i++) {
+			int ret;
+			ret = libusb_get_device_descriptor(gp_devs[i], &gp_descs[i]);
+			if (ret)
+				gp_log (GP_LOG_ERROR, "libusb1", "libusb_get_device_descriptor(%d) returned %d", i, ret);
+		}
+	}
+	time (&gp_devslastchecked);
+	return gp_nrofdevs;
 }
 
 int
@@ -108,40 +121,32 @@ gp_port_library_list (GPPortInfoList *list)
 	int		d, i, i1, i2, unknownint;
 
 	/* generic matcher. This will catch passed XXX,YYY entries for instance. */
-	info.type = GP_PORT_USB;
-	strcpy (info.name, "");
-	strcpy (info.path, "^usb:");
+	gp_port_info_new (&info);
+	gp_port_info_set_type (info, GP_PORT_USB);
+	gp_port_info_set_name (info, "");
+	gp_port_info_set_path (info, "^usb:");
 	CHECK (gp_port_info_list_append (list, info));
 
 	libusb_init (NULL);
-	nrofdevs = load_devicelist ();
-
-	for (d = 0; d < nrofdevs; d++) {
-		struct libusb_device_descriptor desc;
-		int ret;
-
-		ret = libusb_get_device_descriptor(devs[d], &desc);
-		if (ret) {
-			gp_log (GP_LOG_ERROR, "libusb1", "libusb_get_device_descriptor(%d) returned %d", d, ret);
-			continue;
-		}
+	gp_nrofdevs = load_devicelist (NULL);
 
+	for (d = 0; d < gp_nrofdevs; d++) {
 		/* Devices which are definitely not cameras. */
-		if (	(desc.bDeviceClass == LIBUSB_CLASS_HUB)		||
-			(desc.bDeviceClass == LIBUSB_CLASS_HID)		||
-			(desc.bDeviceClass == LIBUSB_CLASS_PRINTER)	||
-			(desc.bDeviceClass == LIBUSB_CLASS_COMM)	||
-			(desc.bDeviceClass == 0xe0)	/* wireless / bluetooth */
+		if (	(gp_descs[d].bDeviceClass == LIBUSB_CLASS_HUB)		||
+			(gp_descs[d].bDeviceClass == LIBUSB_CLASS_HID)		||
+			(gp_descs[d].bDeviceClass == LIBUSB_CLASS_PRINTER)	||
+			(gp_descs[d].bDeviceClass == LIBUSB_CLASS_COMM)	||
+			(gp_descs[d].bDeviceClass == 0xe0)	/* wireless / bluetooth */
 		)
 			continue;
 		/* excepts HUBs, usually the interfaces have the classes, not
 		 * the device */
 		unknownint = 0;
-		for (i = 0; i < desc.bNumConfigurations; i++) {
+		for (i = 0; i < gp_descs[d].bNumConfigurations; i++) {
 			struct libusb_config_descriptor *config;
 			int ret;
 
-			ret = libusb_get_config_descriptor (devs[d], i, &config);
+			ret = libusb_get_config_descriptor (gp_devs[d], i, &config);
 			if (ret) {
 				unknownint++;
 				continue;
@@ -181,31 +186,24 @@ gp_port_library_list (GPPortInfoList *list)
 	/* Redo the same bus/device walk, but now add the ports with usb:x,y notation,
 	 * so we can address all USB devices.
 	 */
-	for (d = 0; d < nrofdevs; d++) {
-		struct libusb_device_descriptor desc;
-		int ret;
-
-		ret = libusb_get_device_descriptor (devs[d], &desc);
-		if (ret) {
-			gp_log (GP_LOG_ERROR, "libusb1", "libusb_get_device_descriptor(%d) returned %d", d, ret);
-			continue;
-		}
+	for (d = 0; d < gp_nrofdevs; d++) {
+		char path[200];
 
 		/* Devices which are definitely not cameras. */
-		if (	(desc.bDeviceClass == LIBUSB_CLASS_HUB)		||
-			(desc.bDeviceClass == LIBUSB_CLASS_HID)		||
-			(desc.bDeviceClass == LIBUSB_CLASS_PRINTER)	||
-			(desc.bDeviceClass == LIBUSB_CLASS_COMM)
+		if (	(gp_descs[d].bDeviceClass == LIBUSB_CLASS_HUB)		||
+			(gp_descs[d].bDeviceClass == LIBUSB_CLASS_HID)		||
+			(gp_descs[d].bDeviceClass == LIBUSB_CLASS_PRINTER)	||
+			(gp_descs[d].bDeviceClass == LIBUSB_CLASS_COMM)
 		)
 			continue;
 		/* excepts HUBs, usually the interfaces have the classes, not
 		 * the device */
 		unknownint = 0;
-		for (i = 0; i < desc.bNumConfigurations; i++) {
+		for (i = 0; i < gp_descs[d].bNumConfigurations; i++) {
 			struct libusb_config_descriptor *config;
 			int ret;
 
-			ret = libusb_get_config_descriptor (devs[d], i, &config);
+			ret = libusb_get_config_descriptor (gp_devs[d], i, &config);
 			if (ret) {
 				gp_log (GP_LOG_ERROR, "libusb1", "libusb_get_config_descriptor(%d) returned %d", d, ret);
 				unknownint++;
@@ -228,20 +226,23 @@ gp_port_library_list (GPPortInfoList *list)
 		/* Note: We do not skip USB storage. Some devices can support both,
 		 * and the Ricoh erronously reports it.
 		 */ 
-		info.type = GP_PORT_USB;
-		strcpy (info.name, "Universal Serial Bus");
-		snprintf (info.path,sizeof(info.path), "usb:%03d,%03d",
-			libusb_get_bus_number (devs[d]),
-			libusb_get_device_address (devs[d])
+		gp_port_info_new (&info);
+		gp_port_info_set_type (info, GP_PORT_USB);
+		gp_port_info_set_name (info, "Universal Serial Bus");
+		snprintf (path,sizeof(path), "usb:%03d,%03d",
+			libusb_get_bus_number (gp_devs[d]),
+			libusb_get_device_address (gp_devs[d])
 		);
+		gp_port_info_set_path (info, path);
 		CHECK (gp_port_info_list_append (list, info));
 	}
 	/* This will only be added if no other device was ever added.
 	 * Users doing "usb:" usage will enter the regular expression matcher case. */
 	if (nrofdevices == 0) {
-		info.type = GP_PORT_USB;
-		strcpy (info.name, "Universal Serial Bus");
-		strcpy (info.path, "usb:");
+		gp_port_info_new (&info);
+		gp_port_info_set_type (info, GP_PORT_USB);
+		gp_port_info_set_name (info, "Universal Serial Bus");
+		gp_port_info_set_path (info, "usb:");
 		CHECK (gp_port_info_list_append (list, info));
 	}
 	libusb_exit (NULL);
@@ -258,14 +259,9 @@ static int gp_port_usb_init (GPPort *port)
 	port->pl->config = port->pl->interface = port->pl->altsetting = -1;
 
 	libusb_init (&port->pl->ctx);
-	libusb_set_debug (port->pl->ctx, 255);
-
 #if 0
-	usb_init ();
-	usb_find_busses ();
-	usb_find_devices ();
+	libusb_set_debug (port->pl->ctx, 255);
 #endif
-
 	return (GP_OK);
 }
 
@@ -277,7 +273,8 @@ gp_port_usb_exit (GPPort *port)
 		free (port->pl);
 		port->pl = NULL;
 	}
-
+	if (gp_devs) libusb_free_device_list (gp_devs, 1);
+	free (gp_descs);
 	return (GP_OK);
 }
 
@@ -311,14 +308,16 @@ gp_port_usb_open (GPPort *port)
 	}
 #endif
 
-	if (ret >= 0) {
-		gp_log (GP_LOG_DEBUG,"libusb1",_("Device has a kernel driver attached (%d), detaching it now."), ret);
+	switch (ret) {
+	case 1: gp_log (GP_LOG_DEBUG,"libusb1",_("Device has a kernel driver attached (%d), detaching it now."), ret);
 		ret = libusb_detach_kernel_driver (port->pl->dh, port->settings.usb.interface);
 		if (ret < 0)
 			gp_port_set_error (port, _("Could not detach kernel driver of camera device."));
 		else
 			port->pl->detached = 1;
-	} else {
+	case 0:	/* not detached */
+		break;
+	default:
 		if (errno != ENODATA) /* ENODATA - just no driver there */
 			gp_port_set_error (port, _("Could not query kernel driver of device."));
 	}
@@ -430,11 +429,15 @@ gp_port_usb_read(GPPort *port, char *bytes, int size)
 {
 	int ret, curread;
 
-	if (!port || !port->pl->dh)
+	if (!port || !port->pl->dh) {
+		gp_log (GP_LOG_ERROR, "libusb1", "gp_port_usb_read: bad parameters");
 		return GP_ERROR_BAD_PARAMETERS;
+	}
 
+	gp_log (GP_LOG_DEBUG, "libusb1", "reading with timeout %d", port->timeout);
 	ret = libusb_bulk_transfer (port->pl->dh, port->settings.usb.inep,
 			     (unsigned char*)bytes, size, &curread, port->timeout);
+	gp_log (GP_LOG_DEBUG, "libusb1", "ret = %d", ret);
         if (ret < 0)
 		return GP_ERROR_IO_READ;
 
@@ -569,9 +572,11 @@ gp_port_usb_update (GPPort *port)
 		port->settings_pending.usb.altsetting
 	);
 
+/* do not overwrite it ... we need to set it.
 	if (port->pl->interface == -1) port->pl->interface = port->settings.usb.interface;
 	if (port->pl->config == -1) port->pl->config = port->settings.usb.config;
 	if (port->pl->altsetting == -1) port->pl->altsetting = port->settings.usb.altsetting;
+*/
 
 	/* The portname can also be changed with the device still fully closed. */
 	memcpy(&port->settings.usb.port, &port->settings_pending.usb.port,
@@ -693,12 +698,13 @@ gp_port_usb_find_ep(libusb_device *dev, int config, int interface, int altsettin
 	if (ret) return -1;
 
 	intf = &confdesc->interface[interface].altsetting[altsetting];
-
 	for (i = 0; i < intf->bNumEndpoints; i++) {
-		if ((intf->endpoint[i].bEndpointAddress & LIBUSB_ENDPOINT_DIR_MASK) == direction &&
-		    (intf->endpoint[i].bmAttributes & LIBUSB_TRANSFER_TYPE_MASK) == type) {
+		if (((intf->endpoint[i].bEndpointAddress & LIBUSB_ENDPOINT_DIR_MASK) == direction) &&
+		    ((intf->endpoint[i].bmAttributes & LIBUSB_TRANSFER_TYPE_MASK) == type)) {
+			unsigned char ret;
+			ret = intf->endpoint[i].bEndpointAddress; /* intf is cleared after next line, copy epaddr! */
 			libusb_free_config_descriptor (confdesc);
-			return intf->endpoint[i].bEndpointAddress;
+			return ret;
 		}
 	}
 	libusb_free_config_descriptor (confdesc);
@@ -766,26 +772,23 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 		return GP_ERROR_BAD_PARAMETERS;
 	}
 
-	nrofdevs = load_devicelist ();
+	gp_nrofdevs = load_devicelist (port->pl->ctx);
 
-	for (d = 0; d < nrofdevs; d++) {
-		struct libusb_device_descriptor desc;
+	for (d = 0; d < gp_nrofdevs; d++) {
 		struct libusb_config_descriptor *confdesc;
 		int ret;
 		int config = -1, interface = -1, altsetting = -1;
 
-		ret = libusb_get_device_descriptor(devs[d], &desc);
-
-		if ((desc.idVendor != idvendor) ||
-		    (desc.idProduct != idproduct))
+		if ((gp_descs[d].idVendor != idvendor) ||
+		    (gp_descs[d].idProduct != idproduct))
 			continue;
 
-		if (busnr && (busnr != libusb_get_bus_number (devs[d])))
+		if (busnr && (busnr != libusb_get_bus_number (gp_devs[d])))
 			continue;
-		if (devnr && (devnr != libusb_get_device_address (devs[d])))
+		if (devnr && (devnr != libusb_get_device_address (gp_devs[d])))
 			continue;
 
-		port->pl->d = devs[d];
+		port->pl->d = gp_devs[d];
 
 		gp_log (GP_LOG_VERBOSE, "libusb1",
 			"Looking for USB device "
@@ -793,9 +796,9 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 			idvendor, idproduct);
 
 		/* Use the first config, interface and altsetting we find */
-		gp_port_usb_find_first_altsetting(devs[d], &config, &interface, &altsetting);
+		gp_port_usb_find_first_altsetting(gp_devs[d], &config, &interface, &altsetting);
 
-		ret = libusb_get_config_descriptor (devs[d], config, &confdesc);
+		ret = libusb_get_config_descriptor (gp_devs[d], config, &confdesc);
 		if (ret)
 			continue;
 
@@ -812,11 +815,11 @@ gp_port_usb_find_device_lib(GPPort *port, int idvendor, int idproduct)
 		port->settings.usb.interface = confdesc->interface[interface].altsetting[altsetting].bInterfaceNumber;
 		port->settings.usb.altsetting = confdesc->interface[interface].altsetting[altsetting].bAlternateSetting;
 
-		port->settings.usb.inep = gp_port_usb_find_ep(devs[d], config, interface, altsetting, LIBUSB_ENDPOINT_IN, LIBUSB_TRANSFER_TYPE_BULK);
-		port->settings.usb.outep = gp_port_usb_find_ep(devs[d], config, interface, altsetting, LIBUSB_ENDPOINT_OUT, LIBUSB_TRANSFER_TYPE_BULK);
-		port->settings.usb.intep = gp_port_usb_find_ep(devs[d], config, interface, altsetting, LIBUSB_ENDPOINT_IN, LIBUSB_TRANSFER_TYPE_INTERRUPT);
+		port->settings.usb.inep  = gp_port_usb_find_ep(gp_devs[d], config, interface, altsetting, LIBUSB_ENDPOINT_IN, LIBUSB_TRANSFER_TYPE_BULK);
+		port->settings.usb.outep = gp_port_usb_find_ep(gp_devs[d], config, interface, altsetting, LIBUSB_ENDPOINT_OUT, LIBUSB_TRANSFER_TYPE_BULK);
+		port->settings.usb.intep = gp_port_usb_find_ep(gp_devs[d], config, interface, altsetting, LIBUSB_ENDPOINT_IN, LIBUSB_TRANSFER_TYPE_INTERRUPT);
 
-		port->settings.usb.maxpacketsize = libusb_get_max_packet_size (devs[d], port->settings.usb.inep);
+		port->settings.usb.maxpacketsize = libusb_get_max_packet_size (gp_devs[d], port->settings.usb.inep);
 		gp_log (GP_LOG_VERBOSE, "libusb1",
 			"Detected defaults: config %d, "
 			"interface %d, altsetting %d, "
@@ -1067,15 +1070,14 @@ gp_port_usb_find_device_by_class_lib(GPPort *port, int class, int subclass, int
 	if (!class)
 		return GP_ERROR_BAD_PARAMETERS;
 
-	nrofdevs = load_devicelist ();
-	for (d = 0; d < nrofdevs; d++) {
-		struct libusb_device_descriptor desc;
+	gp_nrofdevs = load_devicelist (port->pl->ctx);
+	for (d = 0; d < gp_nrofdevs; d++) {
 		struct libusb_config_descriptor *confdesc;
 		int i, ret, config = -1, interface = -1, altsetting = -1;
 
-		if (busnr && (busnr != libusb_get_bus_number (devs[d])))
+		if (busnr && (busnr != libusb_get_bus_number (gp_devs[d])))
 			continue;
-		if (devnr && (devnr != libusb_get_device_address (devs[d])))
+		if (devnr && (devnr != libusb_get_device_address (gp_devs[d])))
 			continue;
 
 		gp_log (GP_LOG_VERBOSE, "gphoto2-port-usb",
@@ -1083,17 +1085,17 @@ gp_port_usb_find_device_by_class_lib(GPPort *port, int class, int subclass, int
 			"(class 0x%x, subclass, 0x%x, protocol 0x%x)...", 
 			class, subclass, protocol);
 
-		ret = gp_port_usb_match_device_by_class(devs[d], class, subclass, protocol, &config, &interface, &altsetting);
+		ret = gp_port_usb_match_device_by_class(gp_devs[d], class, subclass, protocol, &config, &interface, &altsetting);
 		if (!ret)
 			continue;
 
-		port->pl->d = devs[d];
+		port->pl->d = gp_devs[d];
 		gp_log (GP_LOG_VERBOSE, "libusb1",
 			"Found USB class device "
 			"(class 0x%x, subclass, 0x%x, protocol 0x%x)", 
 			class, subclass, protocol);
 
-		ret = libusb_get_config_descriptor (devs[d], config, &confdesc);
+		ret = libusb_get_config_descriptor (gp_devs[d], config, &confdesc);
 		if (ret) continue;
 
 		/* Set the defaults */
@@ -1101,9 +1103,9 @@ gp_port_usb_find_device_by_class_lib(GPPort *port, int class, int subclass, int
 		port->settings.usb.interface = confdesc->interface[interface].altsetting[altsetting].bInterfaceNumber;
 		port->settings.usb.altsetting = confdesc->interface[interface].altsetting[altsetting].bAlternateSetting;
 
-		port->settings.usb.inep = gp_port_usb_find_ep(devs[d], config, interface, altsetting, LIBUSB_ENDPOINT_IN, LIBUSB_TRANSFER_TYPE_BULK);
-		port->settings.usb.outep = gp_port_usb_find_ep(devs[d], config, interface, altsetting, LIBUSB_ENDPOINT_OUT, LIBUSB_TRANSFER_TYPE_BULK);
-		port->settings.usb.intep = gp_port_usb_find_ep(devs[d], config, interface, altsetting, LIBUSB_ENDPOINT_IN, LIBUSB_TRANSFER_TYPE_INTERRUPT);
+		port->settings.usb.inep  = gp_port_usb_find_ep(gp_devs[d], config, interface, altsetting, LIBUSB_ENDPOINT_IN, LIBUSB_TRANSFER_TYPE_BULK);
+		port->settings.usb.outep = gp_port_usb_find_ep(gp_devs[d], config, interface, altsetting, LIBUSB_ENDPOINT_OUT, LIBUSB_TRANSFER_TYPE_BULK);
+		port->settings.usb.intep = gp_port_usb_find_ep(gp_devs[d], config, interface, altsetting, LIBUSB_ENDPOINT_IN, LIBUSB_TRANSFER_TYPE_INTERRUPT);
 		port->settings.usb.maxpacketsize = 0;
 		gp_log (GP_LOG_DEBUG, "libusb1", "inep to look for is %02x", port->settings.usb.inep);
 		for (i=0;i<confdesc->interface[interface].altsetting[altsetting].bNumEndpoints;i++) {
@@ -1120,12 +1122,12 @@ gp_port_usb_find_device_by_class_lib(GPPort *port, int class, int subclass, int
 			port->settings.usb.config,
 			port->settings.usb.interface,
 			port->settings.usb.altsetting,
-			desc.idVendor,
-			desc.idProduct,
+			gp_descs[d].idVendor,
+			gp_descs[d].idProduct,
 			port->settings.usb.inep,
 			port->settings.usb.outep,
 			port->settings.usb.intep
-			);
+		);
 		libusb_free_config_descriptor (confdesc);
 		return GP_OK;
 	}
